/*
    Exotic Fade Filter for VirtualDub.
	by Natan Never 2000 (NNever2000)
    
	I'm searched this filter for months on all internet sites. All fade filter
	that I found was near but not perfect for what I want.

	So I started coding it myself.

	Because I'm a C++ novice, this filter is based on the project files of
	Donald A. Graft's Zoom Filter (that was the first filter that I 
	compiled successfull) and on the Karel Suhajda's DeLogo filter. 
	So, all credits goes to Donald Graft, Avery Lee and Karel Suhajda.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    The original author can be contacted at:

	Natan Never 2000
	nnever2000@yahoo.it.
*/

#define VDEXT_VIDEO_FILTER
#define VDEXT_MAIN

// for save & load bitmap
#define MAXSTR 260


#include <windows.h>
#include <commctrl.h>
#include <crtdbg.h>
#include <assert.h>
#include <stdio.h>
#include <io.h>
#include <direct.h>
#include <math.h>

#include "ScriptInterpreter.h"
#include "ScriptValue.h"
#include "ScriptError.h"

#include "resource.h"
#include "filter.h"

// if logging is defined, all logs are written to C:\delogo.log
//#define LOGGING
//#define LOGLEVEL 3

///////////////////////

/*
#ifdef LOGGING

#define log(x) {logtofile x;}

#define logl(x, y) { if ((x) <= LOGLEVEL) logtofile y; }

#else

#define log(x) {;}

#define logl(x, y) {;}

#endif // LOGGING

///////////////////////

#ifdef LOGGING

void logtofile(char *string, ...) 
{
	FILE *fout;
	va_list argptr;

	fout=fopen("c:\\delogo.log", "a+t");
	if (fout!=NULL) 
	{
		va_start(argptr, string);
		vfprintf(fout, string, argptr);
		fprintf(fout, "\n");
		va_end(argptr);
		fclose(fout);
	}
}

#endif // LOGGING

typedef struct MyBitmap {
	int 	w, h;
	Pixel32 *bitmap;
	char	path[MAXSTR];
} MyBitmap;
*/
enum {
	FILTER_DISABLED	= 0,
	FILTER_FADING	= 1,
	FILTER_LEFT		= 2,
	FILTER_RIGHT	= 3,
	FILTER_TOP		= 4,
	FILTER_BOTTOM	= 5
};

static char *filter_names[]={
	"Disabled",
	"Fading",
	"Left",
	"Right",
	"Top",
	"Bottom"
};

typedef struct MyFilterData {
	
	// fade 1
	int start, frames, filter_mode;
	// fade 2
	int start2, frames2, filter_mode2;
	// fade 3
	int start3, frames3, filter_mode3;
	// fade 4
	int start4, frames4, filter_mode4;
		
	// other

	bool fDebug;


	IFilterPreview *ifp;
	//Resampler *resampler;
	//Resampler::eFilter fmode;
} MyFilterData;

////////////////////

static void GetVirtualDubPath(char *vbpath)
{
	char prog[256];
	LPTSTR ptr;
	GetModuleFileName(NULL, prog, 255);
	GetFullPathName(prog, 255, vbpath, &ptr);
	*ptr = 0;
}

static void GetBitmapFileName(int w, int h, int frame, char *bitmapfilename) 
{
		char framenumber[65];
		_itoa( frame, framenumber, 10 ); 
		char wsize[65];
		_itoa( w, wsize, 10 ); 
		char hsize[65];
		_itoa( h, hsize, 10 ); 
		
		char prog[256];
		//char bitmapfilename[256];
		LPTSTR ptr;

		GetModuleFileName(NULL, prog, 255);
		GetFullPathName(prog, 255, bitmapfilename, &ptr);
		*ptr = 0;
		strcat(bitmapfilename, "temp\\");
		strcat(bitmapfilename, framenumber);
		strcat(bitmapfilename, "-");
		strcat(bitmapfilename, wsize);
		strcat(bitmapfilename, "-");
		strcat(bitmapfilename, hsize);
		strcat(bitmapfilename, ".bin");
}

static bool LoadRawBitmap(int w, int h, int frame, long size, Pixel *src)
{
	FILE *fin;
	bool result;
	char path[256];

	GetBitmapFileName(w, h, frame, path);

	fin=fopen(path, "rb");

	if (fin==NULL) 
	{
		result=false;
	}
	else 
	{
		result=(fread(src, size, 1, fin)>0);
		fclose(fin);
	}

	return result;
}

static bool SaveRawBitmap(int w, int h, int frame, long size, Pixel *src)
{
	// save the bitmap in binary format
	FILE *fout;
	bool result;
	char path[256];
	
/*	// if not exists create the directory
	char vdpath[256];
	GetVirtualDubPath(vdpath);
	strcat(vdpath, "temp\\");
	_mkdir(vdpath);*/

	GetBitmapFileName(w, h, frame, path);

	fout=fopen(path, "wb");

	if (fout==NULL) 
	{
		result=false;
	}
	else 
	{
		result=(fwrite(src, size, 1, fout)>0);			
		fclose(fout);
	}
	return result;
}

static int resize_run(const FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	int frame = fa->pfsi->lCurrentFrame;
	
	// Phase 1
	int start;
	int frames;
	int filter_mode;

	//double percent;
	Pixel *dst, *src;
	unsigned long color;
	int x, y;

	// Default: no fade
	filter_mode = FILTER_DISABLED;

	// if we are in fade 1
	if (frame > mfd->start-mfd->frames && frame <=mfd->start+mfd->frames && mfd->filter_mode!=FILTER_DISABLED)
	{
		start = mfd->start;
		frames = mfd->frames;
		filter_mode = mfd->filter_mode;
	}

	// if we are in fade 2
	if (frame > mfd->start2-mfd->frames2 && frame <=mfd->start2+mfd->frames2 && mfd->filter_mode2!=FILTER_DISABLED)
	{
		start = mfd->start2;
		frames = mfd->frames2;
		filter_mode = mfd->filter_mode2;
	}

	// if we are in fade 3
	if (frame > mfd->start3-mfd->frames3 && frame <=mfd->start3+mfd->frames3 && mfd->filter_mode3!=FILTER_DISABLED)
	{
		start = mfd->start3;
		frames = mfd->frames3;
		filter_mode = mfd->filter_mode3;
	}

	// if we are in fade 4
	if (frame > mfd->start4-mfd->frames4 && frame <=mfd->start4+mfd->frames4 && mfd->filter_mode4!=FILTER_DISABLED)
	{
		start = mfd->start4;
		frames = mfd->frames4;
		filter_mode = mfd->filter_mode4;
	}

	// Begin computing destination birmap proprierties

	/* Ok, only the frames +- the starting framne are used */
	if (filter_mode!=FILTER_DISABLED)
	{
		if (!mfd->fDebug) 
		{
			// save the bitmap in binary format
			bool result = SaveRawBitmap(fa->src.w, fa->src.h, frame, fa->src.size, fa->src.data);			

			// if save is ok then bitmap colour is green else is red
			if (result==false)
			{
				color = (255 << 16) | (0 << 8) | 0; // red color
			}
			else
			{
				color = (0 << 16) | (255 << 8) | 0; // green color
			}

			dst = fa->dst.data;
			src = fa->src.data;

			for (y = 0; y < fa->src.h; y++)
			{
				for (x = 0; x < fa->src.w; x++)
				{
					*dst++ = color;				
				}			
				src	= (Pixel *)((char *)src + fa->src.pitch);
			}
		}
		else
		{
			// ok, here we have to fade

			// first we have to load the two frame to merge
			int frame_in=(frame-start+frames-1)/2+start-frames+1;
			int frame_out=frame_in+frames;

			// output frame is load in destination frame
			dst = fa->dst.data;
			// input frame is load in source frame (I think this is not compliand 
			// to Avery style guide)
			src = fa->src.data;

			// load input frame
			bool result1 = LoadRawBitmap(fa->dst.w, fa->dst.h, frame_in, fa->src.size, src);

			if (result1==false)
			{
				color = (255 << 16) | (255 << 8) | 0; // yellow color

				dst = fa->dst.data;				

				for (y = 0; y < fa->src.h; y++)
				{
					for (x = 0; x < fa->src.w; x++)
					{
						*dst++ = color;				
					}			
				}
			}
			else
			{				
				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						*dst++ = src[x];
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}

			// load output frame in source frame because I'm an idiot
			dst = fa->dst.data;
			src = fa->src.data;

			// load output frame in source frame because I'm an idiot
			bool result2 = LoadRawBitmap(fa->dst.w, fa->dst.h, frame_out, fa->src.size, src);		

			if (result2==false)
			{
				color = (255 << 16) | (255 << 8) | 255; // white color

				src = fa->src.data;

				for (y = 0; y < fa->src.h; y++)
				{
					for (x = 0; x < fa->src.w; x++)
					{
						src[x] = color;				
					}
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
			
			// do merge - destination is just here
				
			dst = fa->dst.data;
			src = fa->src.data;
			//src = fa->src.data;

			if (filter_mode==FILTER_LEFT)
			{
				int new_x = (fa->src.w * (frame - start + frames)) / (frames * 2 + 1 );

				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						if (x<new_x) { *dst++ = src[x]; } else { dst++; }						
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
			if (filter_mode==FILTER_RIGHT)
			{
				int new_x = (fa->src.w * (frame - start + frames)) / (frames * 2 + 1 );

				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						if (x>fa->dst.w-new_x) { *dst++ = src[x]; } else { dst++; }						
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
			if (filter_mode==FILTER_TOP)
			{
				int new_y = (fa->src.h * (frame - start + frames)) / (frames * 2 + 1 );

				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						if (y>fa->dst.h-new_y) { *dst++ = src[x]; } else { dst++; }						
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
			if (filter_mode==FILTER_BOTTOM)
			{
				int new_y = (fa->src.h * (frame - start + frames)) / (frames * 2 + 1 );

				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						if (y<new_y) { *dst++ = src[x]; } else { dst++; }						
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
			if (filter_mode==FILTER_FADING)
			{
				//int new_x = fa->src.w / (frames * 2 + 1 ) * (frame - start + frames);
				int range = frames * 2 + 1;
				int count = frame - start + frames;

				for (y = 0; y < fa->dst.h; y++)
				{
					for (x = 0; x < fa->dst.w; x++)
					{
						int dst_red=*dst>>16 & 255;
						int dst_green=*dst>>8 & 255;
						int dst_blue=*dst & 255;
						
						int src_red=src[x]>>16 & 255;
						int src_green=src[x]>>8 & 255;
						int src_blue=src[x] & 255;;

						int new_red = dst_red + (((src_red-dst_red) * count) / range);
						int new_green = dst_green + (((src_green-dst_green) * count) / range);
						int new_blue = dst_blue + (((src_blue-dst_blue) * count) / range);

						color = (new_red << 16) | (new_green << 8) | new_blue;

						*dst++ = color;
					}								
					src	= (Pixel *)((char *)src + fa->src.pitch);
				}
			}
		}	
	}
	else
	{
		// clone source->destination
		dst = fa->dst.data;
		src = fa->src.data;

		color = (255 << 16) | (0 << 8) | 0; // red color
		for (y = 0; y < fa->src.h; y++)
		{
			for (x = 0; x < fa->src.w; x++)
			{
					*dst++ = src[x];				
			}			
			src	= (Pixel *)((char *)src + fa->src.pitch);
		}
	}

	return 0;
}

static long resize_param(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	if (!mfd) return 0;

/*	if (!mfd->fDebug)
	{
		fa->dst.w		= mfd->new_x;
		fa->dst.h		= mfd->new_y;
	}
	else
	{
		// The debug mode leave the dimensions unchanged
		// simply draw a box around the area on the source bitmap
		// that will be the new bitmap zoomed
		fa->dst.w		= fa->dst.w;
		fa->dst.h		= fa->dst.h;
	}*/

	fa->dst.AlignTo8();

	return FILTERPARAM_SWAP_BUFFERS;
}

static BOOL APIENTRY resizeDlgProc( HWND hDlg, UINT message, UINT wParam, LONG lParam) {
	MyFilterData *mfd = (struct MyFilterData *)GetWindowLong(hDlg, DWL_USER);

    switch (message)
    {
        case WM_INITDIALOG:
			{
				HWND hwndItem;
				int i;

				mfd = (MyFilterData *)lParam;

				SetDlgItemInt(hDlg, IDC_START, mfd->start, FALSE);
				SetDlgItemInt(hDlg, IDC_FRAMES, mfd->frames, FALSE);

				SetDlgItemInt(hDlg, IDC_START2, mfd->start2, FALSE);
				SetDlgItemInt(hDlg, IDC_FRAMES2, mfd->frames2, FALSE);

				SetDlgItemInt(hDlg, IDC_START3, mfd->start3, FALSE);
				SetDlgItemInt(hDlg, IDC_FRAMES3, mfd->frames3, FALSE);

				SetDlgItemInt(hDlg, IDC_START4, mfd->start4, FALSE);
				SetDlgItemInt(hDlg, IDC_FRAMES4, mfd->frames4, FALSE);
				
				CheckDlgButton(hDlg, IDC_DEBUG, mfd->fDebug ? BST_CHECKED : BST_UNCHECKED);
				
				hwndItem = GetDlgItem(hDlg, IDC_FILTER_MODE);
				for(i=0; i<(sizeof filter_names/sizeof filter_names[0]); i++)
					SendMessage(hwndItem, CB_ADDSTRING, 0, (LPARAM)filter_names[i]);
				SendMessage(hwndItem, CB_SETCURSEL, mfd->filter_mode, 0);

				hwndItem = GetDlgItem(hDlg, IDC_FILTER_MODE2);
				for(i=0; i<(sizeof filter_names/sizeof filter_names[0]); i++)
					SendMessage(hwndItem, CB_ADDSTRING, 0, (LPARAM)filter_names[i]);
				SendMessage(hwndItem, CB_SETCURSEL, mfd->filter_mode2, 0);

				hwndItem = GetDlgItem(hDlg, IDC_FILTER_MODE3);
				for(i=0; i<(sizeof filter_names/sizeof filter_names[0]); i++)
					SendMessage(hwndItem, CB_ADDSTRING, 0, (LPARAM)filter_names[i]);
				SendMessage(hwndItem, CB_SETCURSEL, mfd->filter_mode3, 0);

				hwndItem = GetDlgItem(hDlg, IDC_FILTER_MODE4);
				for(i=0; i<(sizeof filter_names/sizeof filter_names[0]); i++)
					SendMessage(hwndItem, CB_ADDSTRING, 0, (LPARAM)filter_names[i]);
				SendMessage(hwndItem, CB_SETCURSEL, mfd->filter_mode4, 0);

				SetWindowLong(hDlg, DWL_USER, (LONG)mfd);
				mfd->ifp->InitButton(GetDlgItem(hDlg, IDC_PREVIEW));
			}
            return (TRUE);

        case WM_COMMAND:                      
			switch(LOWORD(wParam)) {
			case IDOK:
				mfd->ifp->Close();
				EndDialog(hDlg, 0);
				return TRUE;

			case IDCANCEL:
				mfd->ifp->Close();
                EndDialog(hDlg, 1);
                return TRUE;

			case IDC_FRAMES:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_FRAMES, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_FRAMES, mfd->frames, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->frames = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAMES2:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_FRAMES2, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_FRAMES2, mfd->frames2, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->frames2 = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAMES3:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_FRAMES3, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_FRAMES3, mfd->frames3, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->frames3 = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_FRAMES4:
				if (HIWORD(wParam) == EN_UPDATE) {
					long focusx;
					BOOL success;

					focusx = GetDlgItemInt(hDlg, IDC_FRAMES4, &success, FALSE);
					if (!success || focusx > 1024) {
						SetDlgItemInt(hDlg, IDC_FRAMES4, mfd->frames4, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->frames4 = focusx;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_START:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start;
					BOOL success;

					start = GetDlgItemInt(hDlg, IDC_START, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_START, mfd->start, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->start = start;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_START2:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start2;
					BOOL success;

					start2 = GetDlgItemInt(hDlg, IDC_START2, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_START2, mfd->start2, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->start2 = start2;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_START3:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start3;
					BOOL success;

					start3 = GetDlgItemInt(hDlg, IDC_START3, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_START3, mfd->start3, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->start3 = start3;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;

			case IDC_START4:
				if (HIWORD(wParam) == EN_UPDATE) {
					long start4;
					BOOL success;

					start4 = GetDlgItemInt(hDlg, IDC_START4, &success, FALSE);
					if (!success) {
						SetDlgItemInt(hDlg, IDC_START4, mfd->start4, FALSE);
						return TRUE;
					}
					if (mfd != NULL)
					{
						mfd->ifp->UndoSystem();
						mfd->start4 = start4;
						mfd->ifp->RedoSystem();
					}
				}
				return TRUE;
			
			case IDC_PREVIEW:
				mfd->ifp->Toggle(hDlg);
				return TRUE;

			case IDC_FILTER_MODE:
				if (HIWORD(wParam) == CBN_SELCHANGE) {
					mfd->ifp->UndoSystem();
					mfd->filter_mode = SendDlgItemMessage(hDlg, IDC_FILTER_MODE, CB_GETCURSEL, 0, 0);
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_FILTER_MODE2:
				if (HIWORD(wParam) == CBN_SELCHANGE) {
					mfd->ifp->UndoSystem();
					mfd->filter_mode2 = SendDlgItemMessage(hDlg, IDC_FILTER_MODE2, CB_GETCURSEL, 0, 0);
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_FILTER_MODE3:
				if (HIWORD(wParam) == CBN_SELCHANGE) {
					mfd->ifp->UndoSystem();
					mfd->filter_mode3 = SendDlgItemMessage(hDlg, IDC_FILTER_MODE3, CB_GETCURSEL, 0, 0);
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_FILTER_MODE4:
				if (HIWORD(wParam) == CBN_SELCHANGE) {
					mfd->ifp->UndoSystem();
					mfd->filter_mode4 = SendDlgItemMessage(hDlg, IDC_FILTER_MODE4, CB_GETCURSEL, 0, 0);
					mfd->ifp->RedoSystem();
				}
				return TRUE;

			case IDC_DEBUG:
				if (HIWORD(wParam) == BN_CLICKED) {
					BOOL f = IsDlgButtonChecked(hDlg, IDC_DEBUG);

					mfd->ifp->UndoSystem();
					mfd->fDebug = !!f;

					mfd->ifp->RedoSystem();
				}
				return TRUE;

 			case IDHELP:
				{
				char prog[256];
				char path[256];
				LPTSTR ptr;

				GetModuleFileName(NULL, prog, 255);
				GetFullPathName(prog, 255, path, &ptr);
				*ptr = 0;
				strcat(path, "plugins\\ExoticFade.html");
				ShellExecute(hDlg, "open", path, NULL, NULL, SW_SHOWNORMAL);
				return TRUE;
				}

 			case IDC_FLUSH_BUFFER:
				{
				char file_to_remove[256];
				char path[256];
				GetVirtualDubPath(path);
				strcat(path, "temp\\");

				char path2[256];
				GetVirtualDubPath(path2);
				strcat(path2, "temp\\*.bin");

			    struct _finddata_t c_file;
				long hFile;

				/* Find first .c file in current directory */
				if( (hFile = _findfirst( path2 , &c_file )) != -1L )
				{					
					file_to_remove[0]=0;
					strcat(file_to_remove, path);
					strcat(file_to_remove, c_file.name);
					remove(file_to_remove);

					/* Find the rest of the .bin files */
					while( _findnext( hFile, &c_file ) == 0 )
					{					
						file_to_remove[0]=0;
						strcat(file_to_remove, path);
						strcat(file_to_remove, c_file.name);
						remove(file_to_remove);
					}

				   _findclose( hFile );
				}
				return TRUE;
				}
            }
            break;
    }
    return FALSE;
}

static int resize_config(FilterActivation *fa, const FilterFunctions *ff, HWND hWnd) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;
	MyFilterData mfd2 = *mfd;
	int ret;

	mfd->ifp = fa->ifp;

/*	if (mfd->new_x < 16)
		mfd->new_x = 320;
	if (mfd->new_y < 16)
		mfd->new_y = 240;*/

	ret = DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_FILTER_ZOOM),
						 hWnd, resizeDlgProc, (LONG)mfd);

	if (ret)
		*mfd = mfd2;

	return ret;
}

static void resize_string(const FilterActivation *fa, const FilterFunctions *ff, char *buf) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	wsprintf(buf, " (%s)", filter_names[mfd->filter_mode]);
}

static int resize_start(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	// if not exists create the directory
	char vdpath[256];
	GetVirtualDubPath(vdpath);
	strcat(vdpath, "temp\\");
	_mkdir(vdpath);

/*	double dstw = (mfd->startpercent * fa->src.w) / 100.0;
	double dsth = (mfd->startpercent * fa->src.h) / 100.0;

	if (mfd->new_x > fa->src.w || mfd->new_y > fa->src.h)
	{
		MessageBox(NULL,
			"Dimensions of the output frame cannot exceed those of "
			"the input frame. Please correct this and try again."
			,
			"Invalid output window size parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->end < mfd->start)
	{
		MessageBox(NULL,
			"The specified ending frame cannot be before the specified "
			"starting frame. Please correct this and try again."
			,
			"Invalid ending frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startpercent2 != 0 && mfd->start2 < mfd->end)
	{
		MessageBox(NULL,
			"The specified phase 2 starting frame cannot be before the "
			"specified phase 1 ending frame. Please correct this and try again."
			,
			"Invalid starting frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startpercent2 != 0 && mfd->end2 < mfd->start2)
	{
		MessageBox(NULL,
			"The specified phase 2 ending frame cannot be before the "
			"specified phase 2 starting frame. Please correct this and try again."
			,
			"Invalid ending frame parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (mfd->startfocusx > fa->src.w - 1 || mfd->startfocusy > fa->src.h - 1)
	{
		MessageBox(NULL,
			"The focus point must be within the dimensions of "
			"the input frame. Please correct this and try again."
			,
			"Invalid focus parameter",
			MB_ICONEXCLAMATION);
		return 1;
	}
	if (dstw<16 || dsth<16)
	{
		MessageBox(NULL,
			"The resampler requires the scaled output area to be at least "
			"16 x 16 pixels. Please correct this and try again."
			,
			"Invalid resampler configuration",
			MB_ICONEXCLAMATION);
		return 1;
	}

	switch(mfd->filter_mode) {
	case FILTER_NONE:			mfd->fmode = Resampler::eFilter::kPoint; break;
	case FILTER_BILINEAR:		mfd->fmode = Resampler::eFilter::kLinearInterp; break;
	case FILTER_BICUBIC:		mfd->fmode = Resampler::eFilter::kCubicInterp; break;
	case FILTER_TABLEBILINEAR:	mfd->fmode = Resampler::eFilter::kLinearDecimate; break;
	case FILTER_TABLEBICUBIC:	mfd->fmode = Resampler::eFilter::kCubicDecimate; break;
	}

	mfd->resampler = new Resampler();

	mfd->resampler->Init(mfd->fmode, mfd->fmode, dstw, dsth, fa->src.w, fa->src.h);*/

	return 0;
}

static int resize_stop(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

//	delete mfd->resampler;	mfd->resampler = NULL;

	return 0;
}

static void resize_script_config(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc)
{
	FilterActivation *fa = (FilterActivation *)lpVoid;

	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->frames			= argv[0].asInt();
	mfd->start			= argv[1].asInt();
	mfd->filter_mode	= argv[2].asInt();
	mfd->frames2		= argv[3].asInt();
	mfd->start2			= argv[4].asInt();
	mfd->filter_mode2	= argv[5].asInt();
	mfd->frames3		= argv[6].asInt();
	mfd->start3			= argv[7].asInt();
	mfd->filter_mode3	= argv[8].asInt();
	mfd->frames4		= argv[9].asInt();
	mfd->start4			= argv[10].asInt();
	mfd->filter_mode4	= argv[11].asInt();
	mfd->fDebug			= argv[12].asInt();
}

static ScriptFunctionDef resize_func_defs[]={
	{ (ScriptFunctionPtr)resize_script_config, "Config", "0iiiiiiiiiiiiiiii" },
	{ NULL },
};

static CScriptObject resize_obj={
	NULL, resize_func_defs
};

static bool resize_script_line(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen)
{
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;	

	_snprintf(buf, buflen, "Config(%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d)",
			  mfd->frames,  mfd->start, mfd->filter_mode,
			  mfd->frames2,  mfd->start2, mfd->filter_mode2,
			  mfd->frames3,  mfd->start3, mfd->filter_mode3,
			  mfd->frames4,  mfd->start4, mfd->filter_mode4,
			  mfd->fDebug);
	return true;
}

int resize_init(FilterActivation *fa, const FilterFunctions *ff);

FilterDefinition filterDef_resize={
	0,0,NULL,
	"Exotic Fade (1.0 beta5)",
	"Exotic Fade (based on Zoom 1.2 sources by Donald Graft)."
#ifdef USE_ASM
			"\n[Assembly optimized] [FPU optimized] [MMX optimized]"
#endif
			,
	"NNever2000",
	NULL,
	sizeof(MyFilterData),
	resize_init,
	NULL,
	resize_run,
	resize_param,
	resize_config,
	resize_string,
	resize_start,
	resize_stop,

	&resize_obj,
	resize_script_line,
};

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
	if (!(fd_tutorial = ff->addFilter(fm, &filterDef_resize, sizeof(FilterDefinition))))
		return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

	return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
	ff->removeFilter(fd_tutorial);
}

int resize_init(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	INITIALIZE_VTBLS;

	mfd->frames = 0;
	mfd->start = 0;
	mfd->filter_mode = FILTER_DISABLED;

	mfd->frames2 = 0;
	mfd->start2 = 0;
	mfd->filter_mode2 = FILTER_DISABLED;

	mfd->frames3 = 0;
	mfd->start3 = 0;
	mfd->filter_mode3 = FILTER_DISABLED;

	mfd->frames4 = 0;
	mfd->start4 = 0;
	mfd->filter_mode4 = FILTER_DISABLED;

	mfd->fDebug = false;
	return 0;
}


/*
void LoadBitmapFile(MyBitmap *bitmap, HWND hdlg, BOOL warnings) 
{
	logl(5, ("LoadBitmapFile() ->"));

	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	FILE *fin = NULL; // jfp 20/05/02
	Pixel32 *bufuk;
	long y, x;
	byte *bmpline, *bmpc;
	byte r, g, b;

	enum restype { 
		BMP_OK, // OK
		BMP_NONAME, // No name given
		BMP_NOFILE, // File can not be open
		BMP_WRONG, // File seems not be BMP file
		BMP_ERROR, // Error reading contents of file
		BMP_BADFMT, // Bad format of file - must be uncompressed 24-bit RGB
		BMP_MEMERR // Not enough memory to load image
	} result;

	DeinitBitmap(bitmap);

	result=BMP_OK;
	if (bitmap->path[0]==0) 
	{
		result=BMP_NONAME;
	}
	else if ((fin=fopen(bitmap->path, "rb"))==NULL) 
	{
		result=BMP_NOFILE;
	}
	else if (fread(&bmfh, 1, sizeof(bmfh), fin)!=sizeof(bmfh)) 
	{
		result=BMP_ERROR;
	}
	else if (fread(&bmih, 1, sizeof(bmih), fin)!=sizeof(bmih)) 
	{
		result=BMP_ERROR;
	}
	else if ((bmfh.bfType!=0x4D42) || (bmfh.bfReserved1!=0) || (bmfh.bfReserved2!=0)) 
	{
		result=BMP_WRONG;
	}
	else if ((bmih.biSize<sizeof(bmih)) || (bmih.biHeight==0) || (bmih.biWidth==0)) 
	{
		result=BMP_WRONG;
	}
	else if ((bmih.biBitCount!=24) || (bmih.biCompression!=BI_RGB) || (bmih.biPlanes!=1) || (bmih.biHeight<0)) 
	{
		result=BMP_BADFMT;
	}
	else if (fseek(fin, 0, SEEK_END)) 
	{
		result=BMP_ERROR;
// I just hope nobody will create a bitmap bigger than 2 GB ;-)
	}
	else if (ftell(fin)-bmfh.bfOffBits<(ulong)(3*bmih.biHeight*bmih.biWidth)) 
	{
		result=BMP_ERROR;
	}
	else if (fseek(fin, bmfh.bfOffBits, SEEK_SET)) 
	{
		result=BMP_ERROR;
	}
	else if (NULL == (bitmap->bitmap = new Pixel32[bmih.biHeight*bmih.biWidth])) 
	{
		result=BMP_MEMERR;
	}
	else if (!(bmpline = new byte[bmih.biWidth*3])) 
	{
		result=BMP_MEMERR;
	}
	else 
	{
		logl(3, ("New bitmap = %d", bitmap->bitmap));
		bitmap->w=bmih.biWidth;
		bitmap->h=bmih.biHeight;

		bufuk=bitmap->bitmap;
		y=bmih.biHeight;
		do 
		{
			if (fread(bmpline, (bmih.biWidth*3+3)&~3, 1, fin)!=1) 
			{
				result=BMP_WRONG;
				y=1; // to get out of the loop
			}
			else
			{
				bmpc=bmpline;
				x=bmih.biWidth;
				do 
				{
					r=*bmpc++;
					g=*bmpc++;
					b=*bmpc++;
					*bufuk++=(((long)b)<<16) | (((long)g)<<8) | ((long)r);
				} while (--x);
			}
		} while (--y);
		logl(3, ("Delete bmpline(%d)", bmpline));
		delete [] bmpline;
	}
	
	if (fin!=NULL) fclose(fin);

	if (result!=BMP_OK) 
	{
		DeinitBitmap(bitmap);
		if (warnings) 
		{
			switch (result) 
			{
			case BMP_NONAME:
				MessageBox(hdlg, "Missing bitmap file name", "Bitmap load error", MB_OK);
				break;
				 
			case BMP_NOFILE:
				MessageBox(hdlg, "Bitmap file not found", "Bitmap load error", MB_OK);
				break;

			case BMP_ERROR:
				MessageBox(hdlg, "Error reading bitmap file", "Bitmap load error", MB_OK);
				break;

			case BMP_WRONG:
				MessageBox(hdlg, "Selected file is corrupted or it is not a bitmap file", "Bitmap load error", MB_OK);
				break;

			case BMP_BADFMT:
				MessageBox(hdlg, "Only uncompressed 24-bit RGB bottom-up bitmaps are supported", "Bitmap load error", MB_OK);
				break;

			case BMP_MEMERR:
				MessageBox(hdlg, "Not enough memory to load bitmap file", "Bitmap load error", MB_OK);
				break;
			}
		}
	}
	logl(5, ("<- LoadBitmapFile()"));
}

///////////////////////////////////////////////////////////////////////////

void SaveBitmap(MyFilterData *mfd, MyBitmap *bmpptr) 
{
	logl(5, ("SaveBitmapFile() ->"));

	OPENFILENAME ofn;
	BITMAPFILEHEADER bmfh;
	BITMAPINFOHEADER bmih;
	FILE *fout;
	Pixel32 *bufuk;
	long y, x;
	byte *bmpline, *bmpc;
	bool result;

	if (bmpptr->bitmap==NULL) 
	{
		return;
	}

	bmfh.bfType=0x4D42;
	bmfh.bfSize=sizeof(bmfh)+sizeof(bmih)+bmpptr->w*bmpptr->h*3;
	bmfh.bfReserved1=0;
	bmfh.bfReserved2=0;
	bmfh.bfOffBits=sizeof(bmfh)+sizeof(bmih);

	bmih.biSize=sizeof(bmih);
	bmih.biWidth=bmpptr->w;
	bmih.biHeight=bmpptr->h;
	bmih.biPlanes=1;
	bmih.biBitCount=24;
	bmih.biCompression=BI_RGB;
	bmih.biSizeImage=0;
	bmih.biXPelsPerMeter=3780;
	bmih.biYPelsPerMeter=3780;
	bmih.biClrUsed=0;
	bmih.biClrImportant=0;


	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=mfd->hdlg;
	ofn.lpstrFilter="Bitmap files (*.bmp)\0*.bmp\0\0";
	ofn.lpstrCustomFilter=NULL;
	ofn.nFilterIndex=1;
	ofn.lpstrFile=bmpptr->path;
	ofn.nMaxFile=MAXSTR;
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrTitle="Save bitmap";
	ofn.Flags=OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
	ofn.lpstrDefExt="bmp";
	ofn.lpstrFileTitle=NULL;

	if (NULL == (bmpline = new byte[(bmpptr->w*3+3)&~3])) 
	{
		MessageBox(mfd->hdlg, "Not enough memory to save bitmap!", "Bitmap save error", MB_OK);
	}
	else 
	{
		logl(3, ("New bmpline = %d", bmpline));
		if (GetSaveFileName(&ofn)) 
		{
			fout=fopen(ofn.lpstrFile, "wb");
			if (fout==NULL) 
			{
				MessageBox(mfd->hdlg, "Can't create file!", "Bitmap save error", MB_OK);
			}
			else 
			{
				result=(fwrite(&bmfh, sizeof(bmfh), 1, fout)>0);

				if (result) 
				{
					result=(fwrite(&bmih, 1, sizeof(bmih), fout)>0);
				}

				if (result) 
				{
					bufuk=bmpptr->bitmap;
					y=bmpptr->h;
					do 
					{
						x=bmpptr->w;
						bmpc=bmpline;
						do 
						{
							*bmpc++=(byte)(*bufuk & 255);
							*bmpc++=(byte)((*bufuk >> 8) & 255);
							*bmpc++=(byte)((*bufuk >> 16) & 255);
							bufuk++;
						} while (--x);
						if (fwrite(bmpline, (bmpptr->w*3+3)&~3, 1, fout)<1) 
						{
							result=false;
							y=1;
						}
					} while (--y);
				}
				if (!result) 
				{
					MessageBox(mfd->hdlg, "Error writing bitmap file", "Bitmap save error", MB_OK);
				}
				fclose(fout);
			}
		}
		logl(3, ("Delete bmpline(%d)", bmpline));
		delete [] bmpline;
	}
	logl(5, ("<- SaveBitmapFile()"));
}

void DeinitBitmap(MyBitmap *ptr) 
{
	logl(5, ("DeinitBitmap() ->"));
	logl(3, ("Delete Bitmap(%d)", ptr->bitmap));
	delete[] ptr->bitmap;
	ptr->bitmap=NULL;
	ptr->w=0;
	ptr->h=0;
	logl(5, ("<- DeinitBitmap()"));
}*/

///////////////////////////////////////////////////////////////////////////

